@@ -1,6 +1,6 @@
 /*
  *  oftc-ircservices: an exstensible and flexible IRC Services package
- *  chanserv.c: A C implementation of Chanell Services
+ *  chanserv.c: A C implementation of Channel Services
  *
  *  Copyright (C) 2006 The OFTC Coding department
  *
@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: chanserv.c 912 2007-05-03 16:03:13Z stu $
+ *  $Id: chanserv.c 1043 2007-05-18 09:24:56Z swalsh $
  */
 
 #include "stdinc.h"
@@ -30,8 +30,8 @@ static dlink_node *cs_cmode_hook;
 static dlink_node *cs_join_hook;
 static dlink_node *cs_channel_destroy_hook;
 static dlink_node *cs_channel_create_hook;
-static dlink_node *cs_on_nick_drop_hook;
 static dlink_node *cs_on_topic_change_hook;
+static dlink_node *cs_on_burst_done_hook;
 
 static dlink_list channel_limit_list = { NULL, NULL, 0 };
 static dlink_list channel_expireban_list = { NULL, NULL, 0 };
@@ -43,8 +43,8 @@ static void *cs_on_cmode_change(va_list);
 static void *cs_on_client_join(va_list);
 static void *cs_on_channel_destroy(va_list);
 static void *cs_on_channel_create(va_list);
-static void *cs_on_nick_drop(va_list);
 static void *cs_on_topic_change(va_list);
+static void *cs_on_burst_done(va_list);
 
 static void m_register(struct Service *, struct Client *, int, char *[]);
 static void m_help(struct Service *, struct Client *, int, char *[]);
@@ -70,6 +70,7 @@ static void m_set_floodserv(struct Service *, struct Client *, int, char *[]);
 static void m_set_mlock(struct Service *, struct Client *, int, char *[]);
 static void m_set_autoop(struct Service *, struct Client *, int, char *[]);
 static void m_set_autovoice(struct Service *, struct Client *, int, char *[]);
+static void m_set_leaveops(struct Service *, struct Client *, int, char *[]);
 
 static void m_akick_add(struct Service *, struct Client *, int, char *[]);
 static void m_akick_list(struct Service *, struct Client *, int, char *[]);
@@ -83,6 +84,8 @@ static void m_clear_users(struct Service *, struct Client *, int, char *[]);
 
 static void m_op(struct Service *, struct Client *, int, char *[]);
 static void m_deop(struct Service *, struct Client *, int, char *[]);
+static void m_voice(struct Service *, struct Client *, int, char *[]);
+static void m_devoice(struct Service *, struct Client *, int, char *[]);
 static void m_invite(struct Service *, struct Client *, int, char *[]);
 static void m_unban(struct Service *, struct Client *, int, char *[]);
 static void m_list(struct Service *, struct Client *, int, char *[]);
@@ -137,6 +140,8 @@ static struct ServiceMessage set_sub[] = {
     CS_HELP_SET_AUTOOP_SHORT, CS_HELP_SET_AUTOOP_LONG, m_set_autoop },
   { NULL, "AUTOVOICE", 0, 1, 2, SFLG_KEEPARG|SFLG_CHANARG, MASTER_FLAG,
     CS_HELP_SET_AUTOVOICE_SHORT, CS_HELP_SET_AUTOVOICE_LONG, m_set_autovoice },
+  { NULL, "LEAVEOPS", 0, 1, 2, SFLG_KEEPARG|SFLG_CHANARG, MASTER_FLAG,
+    CS_HELP_SET_LEAVEOPS_SHORT, CS_HELP_SET_LEAVEOPS_LONG, m_set_leaveops },
   { NULL, NULL, 0, 0, 0, SFLG_KEEPARG|SFLG_CHANARG, 0, 0, 
     SFLG_KEEPARG|SFLG_CHANARG, NULL } 
 };
@@ -198,6 +203,16 @@ static struct ServiceMessage deop_msgtab = {
   CS_HELP_DEOP_SHORT, CS_HELP_DEOP_LONG, m_deop
 };
 
+static struct ServiceMessage voice_msgtab = {
+  NULL, "VOICE", 0, 1, 2, SFLG_CHANARG, MEMBER_FLAG, 
+  CS_HELP_VOICE_SHORT, CS_HELP_VOICE_LONG, m_voice
+};
+
+static struct ServiceMessage devoice_msgtab = {
+  NULL, "DEVOICE", 0, 1, 2, SFLG_CHANARG, MEMBER_FLAG, 
+  CS_HELP_DEVOICE_SHORT, CS_HELP_DEVOICE_LONG, m_devoice
+};
+
 static struct ServiceMessage unban_msgtab = {
   NULL, "UNBAN", 0, 1, 1, SFLG_CHANARG, MEMBER_FLAG, 
   CS_HELP_UNBAN_SHORT, CS_HELP_UNBAN_LONG, m_unban
@@ -245,7 +260,7 @@ static struct ServiceMessage unforbid_msgtab = {
   CS_HELP_UNFORBID_SHORT, CS_HELP_UNFORBID_LONG, m_unforbid
 };
 
-INIT_MODULE(chanserv, "$Revision: 912 $")
+INIT_MODULE(chanserv, "$Revision: 1043 $")
 {
   chanserv = make_service("ChanServ");
   clear_serv_tree_parse(&chanserv->msg_tree);
@@ -265,6 +280,8 @@ INIT_MODULE(chanserv, "$Revision: 912 $")
   mod_add_servcmd(&chanserv->msg_tree, &info_msgtab);
   mod_add_servcmd(&chanserv->msg_tree, &op_msgtab);
   mod_add_servcmd(&chanserv->msg_tree, &deop_msgtab);
+  mod_add_servcmd(&chanserv->msg_tree, &voice_msgtab);
+  mod_add_servcmd(&chanserv->msg_tree, &devoice_msgtab);
   mod_add_servcmd(&chanserv->msg_tree, &invite_msgtab);
   mod_add_servcmd(&chanserv->msg_tree, &clear_msgtab);
   mod_add_servcmd(&chanserv->msg_tree, &unban_msgtab);
@@ -281,8 +298,8 @@ INIT_MODULE(chanserv, "$Revision: 912 $")
 
   cs_channel_create_hook = install_hook(on_channel_created_cb, 
       cs_on_channel_create);
-  cs_on_nick_drop_hook = install_hook(on_nick_drop_cb, cs_on_nick_drop);
   cs_on_topic_change_hook = install_hook(on_topic_change_cb, cs_on_topic_change);
+  cs_on_burst_done_hook = install_hook(on_burst_done_cb, cs_on_burst_done);
 
   eventAdd("process channel autolimits", process_limit_list, NULL, 10);
   eventAdd("process channel expirebans", process_expireban_list, NULL, 10);
@@ -294,8 +311,8 @@ CLEANUP_MODULE
   uninstall_hook(on_join_cb, cs_on_client_join);
   uninstall_hook(on_channel_destroy_cb, cs_on_channel_destroy);
   uninstall_hook(on_channel_created_cb, cs_on_channel_create);
-  uninstall_hook(on_nick_drop_cb, cs_on_nick_drop);
   uninstall_hook(on_topic_change_cb, cs_on_topic_change);
+  uninstall_hook(on_burst_done_cb, cs_on_burst_done);
 
   serv_clear_messages(chanserv);
 
@@ -318,7 +335,7 @@ process_limit_list(void *param)
   {
     struct Channel *chptr = ptr->data;
 
-    if(chptr->limit_time > CurrentTime)
+    if(chptr->limit_time < CurrentTime)
     {
       int limit;
 
@@ -527,15 +544,15 @@ m_info(struct Service *service, struct Client *client,
   reply_user(service, service, client, CS_INFO_OPTION, "EXPIREBANS",
       regchptr->expirebans ? "ON" : "OFF");
 
-  reply_user(service, service, client, CS_INFO_OPTION, "EXPIREBANS",
-      regchptr->expirebans ? "ON" : "OFF");
-
   reply_user(service, service, client, CS_INFO_OPTION, "AUTOOP",
       regchptr->autoop ? "ON" : "OFF");
 
   reply_user(service, service, client, CS_INFO_OPTION, "AUTOVOICE",
       regchptr->autovoice ? "ON" : "OFF");
 
+  reply_user(service, service, client, CS_INFO_OPTION, "LEAVEOPS",
+      regchptr->leaveops ? "ON" : "OFF");
+
   free_regchan(regchptr);
 }
 
@@ -553,7 +570,7 @@ m_access_add(struct Service *service, struct Client *client,
 {
   struct Channel *chptr;
   struct RegChannel *regchptr;
-  struct ChanAccess *access;
+  struct ChanAccess *access, *oldaccess;
   unsigned int account, level;
   char *level_added = "MEMBER";
 
@@ -592,6 +609,21 @@ m_access_add(struct Service *service, struct Client *client,
   access->channel = regchptr->id;
   access->account = account;
   access->level   = level;
+
+  if((oldaccess = db_find_chanaccess(access->channel, access->account)) != NULL)
+  {
+    if(oldaccess->level == MASTER_FLAG && db_get_num_masters(regchptr->id) <= 1)
+    {
+      reply_user(service, service, client, CS_ACCESS_NOMASTERS, parv[2],
+          regchptr->channel);
+      if(chptr == NULL)
+        free_regchan(regchptr);
+      free_chanaccess(oldaccess);
+      return;
+    }
+    db_list_del_index(DELETE_CHAN_ACCESS, access->account, access->channel);
+    free_chanaccess(oldaccess);
+  }
  
   if(db_list_add(CHACCESS_LIST, access))
   {
@@ -668,7 +700,7 @@ static void
 m_access_list(struct Service *service, struct Client *client,
     int parc, char *parv[])
 {
-  struct ChanAccess *access;
+  struct ChanAccess *access = NULL;
   struct Channel *chptr;
   struct RegChannel *regchptr;
   void *handle, *first;
@@ -682,6 +714,9 @@ m_access_list(struct Service *service, struct Client *client,
   if (handle == NULL)
   {
     reply_user(service, service, client, CS_ACCESS_LISTEND, regchptr->channel);
+    free_chanaccess(access);
+    if(chptr == NULL)
+      free_regchan(regchptr);
     return;
   }
 
@@ -715,7 +750,6 @@ m_access_list(struct Service *service, struct Client *client,
   }
 
   free_chanaccess(access);
-
   db_list_done(first);
   reply_user(service, service, client, CS_ACCESS_LISTEND, regchptr->channel);
 
@@ -824,8 +858,9 @@ m_set_topic(struct Service *service, struct Client *client,
   m_set_string(service, client, parv[1], "TOPIC", SET_CHAN_TOPIC, value, 
       &regchptr->topic, parc);
 
-  if(topic != NULL && strncmp(topic, regchptr->topic, 
-        LIBIO_MAX(strlen(topic), strlen(regchptr->topic))) != 0)
+  if(topic != NULL && (regchptr->topic == NULL || 
+        strncmp(topic, regchptr->topic, 
+          LIBIO_MAX(strlen(topic), strlen(regchptr->topic)))) != 0)
     changetopic = TRUE;
 
   if(changetopic || ((topic == NULL && (regchptr->topic != NULL)) && (chptr != NULL)))
@@ -917,6 +952,14 @@ m_set_autovoice(struct Service *service, struct Client *client,
      "AUTOVOICE");
 }
 
+static void
+m_set_leaveops(struct Service *service, struct Client *client,
+    int parc, char *parv[])
+{
+  m_set_flag(service, client, parv[1], parv[2], SET_CHAN_LEAVEOPS, 
+     "LEAVEOPS");
+}
+
 static void
 m_set_autolimit(struct Service *service, struct Client *client,
     int parc, char *parv[])
@@ -992,13 +1035,8 @@ m_set_floodserv(struct Service *service, struct Client *client,
 
     floodserv = find_client(fsname);
 
-    if(floodserv != NULL)
+    if(floodserv != NULL && IsMember(floodserv, chptr))
       part_channel(floodserv, chptr->chname, "Unset");
-    else
-    {
-      ilog(L_INFO, "%s SET %s OFF for %s but FloodServ isn't loaded",
-        client->name, fsname, chptr->chname);
-    }
   }
 }
 
@@ -1076,7 +1114,7 @@ static void
 m_akick_list(struct Service *service, struct Client *client,
     int parc, char *parv[])
 {
-  struct ServiceBan *akick;
+  struct ServiceBan *akick = NULL;
   void *handle, *first;
   int i = 1;
   struct Channel *chptr;
@@ -1137,6 +1175,9 @@ m_akick_del(struct Service *service, struct Client *client,
   else
     ret = db_list_del(DELETE_AKICK_ACCOUNT, regchptr->id, parv[2]);
 
+  if(chptr == NULL)
+    free_regchan(regchptr);
+
   reply_user(service, service, client, CS_AKICK_DEL, ret);
 }
 
@@ -1325,7 +1366,7 @@ m_op(struct Service *service, struct Client *client, int parc, char *parv[])
 
   if(target == NULL || (ms = find_channel_link(target, chptr)) == NULL)
   {
-    reply_user(service, service, client, CS_NOT_ON_CHAN, target->name, parv[1]);
+    reply_user(service, service, client, CS_NOT_ON_CHAN, parv[2], parv[1]);
     return;
   }
 
@@ -1334,6 +1375,8 @@ m_op(struct Service *service, struct Client *client, int parc, char *parv[])
     op_user(service, chptr, target);
     reply_user(service, service, client, CS_OP, target->name, parv[1]);
   }
+  else
+    reply_user(service, service, client, CS_ALREADY_OP, target->name, parv[1]);
 }
 
 static void
@@ -1369,6 +1412,104 @@ m_deop(struct Service *service, struct Client *client, int parc, char *parv[])
     deop_user(service, chptr, target);
     reply_user(service, service, client, CS_DEOP, target->name, parv[1]);
   }
+  else
+    reply_user(service, service, client, CS_NOT_OP, target->name, parv[1]);
+}
+
+static void
+m_voice(struct Service *service, struct Client *client, int parc, char *parv[])
+{
+  struct Channel    *chptr;
+  struct RegChannel *regchptr;
+  struct Client     *target;
+  struct Membership *ms;
+  struct ChanAccess *access;
+
+  chptr = hash_find_channel(parv[1]);
+
+  if(chptr == NULL)
+  {
+    reply_user(service, service, client, CS_CHAN_NOT_USED, parv[1]);
+    return;
+  }
+
+  regchptr = chptr->regchan;
+  access = db_find_chanaccess(regchptr->id, client->nickname->id);
+
+  if(parv[2] == NULL)
+    target = client;
+  else if(access->level < CHANOP_FLAG)
+  {
+    reply_user(service, service, client, CS_NO_VOICE_OTHERS, parv[1]);
+    free_chanaccess(access);
+    return;
+  }
+  else
+    target = find_client(parv[2]);
+
+  free_chanaccess(access);
+
+  if(target == NULL || (ms = find_channel_link(target, chptr)) == NULL)
+  {
+    reply_user(service, service, client, CS_NOT_ON_CHAN, parv[2], parv[1]);
+    return;
+  }
+
+  if(!has_member_flags(ms, CHFL_VOICE))
+  {
+    voice_user(service, chptr, target);
+    reply_user(service, service, client, CS_VOICE, target->name, parv[1]);
+  }
+  else
+    reply_user(service, service, client, CS_ALREADY_VOICE, target->name, parv[1]);
+}
+
+static void
+m_devoice(struct Service *service, struct Client *client, int parc, char *parv[])
+{
+  struct Channel    *chptr;
+  struct RegChannel *regchptr;
+  struct Client     *target;
+  struct Membership *ms;
+  struct ChanAccess *access;
+
+  chptr = hash_find_channel(parv[1]);
+
+  if(chptr == NULL)
+  {
+    reply_user(service, service, client, CS_CHAN_NOT_USED, parv[1]);
+    return;
+  }
+
+  regchptr = chptr->regchan;
+  access = db_find_chanaccess(regchptr->id, client->nickname->id);
+
+  if(parv[2] == NULL)
+    target = client;
+  else if(access->level < CHANOP_FLAG)
+  {
+    reply_user(service, service, client, CS_NO_DEVOICE_OTHERS, parv[1]);
+    free_chanaccess(access);
+    return;
+  }
+  else
+    target = find_client(parv[2]);
+
+  free_chanaccess(access);
+
+  if(target == NULL || (ms = find_channel_link(target, chptr)) == NULL)
+  {
+    reply_user(service, service, client, CS_NOT_ON_CHAN, parv[2], parv[1]);
+    return;
+  }
+
+  if(has_member_flags(ms, CHFL_VOICE))
+  {
+    devoice_user(service, chptr, target);
+    reply_user(service, service, client, CS_DEVOICE, target->name, parv[1]);
+  }
+  else
+    reply_user(service, service, client, CS_NOT_VOICE, target->name, parv[1]);
 }
 
 static void
@@ -1461,23 +1602,29 @@ m_set_string(struct Service *service, struct Client *client,
 
     if(response == NULL)
       response = "Not set";
-    reply_user(service, service, client, CS_SET_VALUE, field, response, 
-        regchptr->channel);
+    if(client != NULL)
+    {
+      reply_user(service, service, client, CS_SET_VALUE, field, response, 
+          regchptr->channel);
+    }
     if(chptr == NULL)
       free_regchan(regchptr);
     return TRUE;
   }
 
-  if(ircncmp(value, "-", strlen(value)) == 0)
+  if(value != NULL && ircncmp(value, "-", strlen(value)) == 0)
     value = NULL;
 
   if(db_set_string(query, regchptr->id, value))
   {
-    reply_user(service, service, client, CS_SET_SUCCESS, field, 
-        value == NULL ? "Not set" : value, regchptr->channel);
-    ilog(L_DEBUG, "%s (%s@%s) changed %s of %s to %s", 
-      client->name, client->username, client->host, field, regchptr->channel, 
-      value);
+    if(client != NULL)
+    {
+      reply_user(service, service, client, CS_SET_SUCCESS, field, 
+          value == NULL ? "Not set" : value, regchptr->channel);
+      ilog(L_DEBUG, "%s (%s@%s) changed %s of %s to %s", 
+          client->name, client->username, client->host, field, regchptr->channel, 
+          value);
+    }
 
     if(value != NULL)
       *param = replace_string(*param, value);
@@ -1491,7 +1638,7 @@ m_set_string(struct Service *service, struct Client *client,
 
     return TRUE;
   }
-  else
+  else if(client != NULL)
     reply_user(service, service, client, CS_SET_FAILED, field,
         value == NULL ? "Not set" : value, regchptr->channel);
   
@@ -1553,6 +1700,8 @@ m_set_flag(struct Service *service, struct Client *client,
   {
     reply_user(service, service, client, CS_SET_VALUE, flagname,
         on ? "ON" : "OFF", channel);
+    if(chptr == NULL)
+      free_regchan(regchptr);
     return -1;
   }
 
@@ -1587,6 +1736,9 @@ m_set_flag(struct Service *service, struct Client *client,
       case SET_CHAN_AUTOVOICE:
         regchptr->autovoice = on;
         break;
+      case SET_CHAN_LEAVEOPS:
+        regchptr->leaveops = on;
+        break;
     }
     if (chptr == NULL)
       free_regchan(regchptr);
@@ -1625,7 +1777,7 @@ m_sudo(struct Service *service, struct Client *client, int parc, char *parv[])
   MyFree(newparv[2]);
   MyFree(newparv);
 
-  client->access = MASTER_FLAG;
+  client->access = ADMIN_FLAG;
 }
 
 static void
@@ -1847,6 +1999,9 @@ cs_on_client_join(va_list args)
       level = CHIDENTIFIED_FLAG;
     else
       level = access->level;
+
+    if(access != NULL)
+      free_chanaccess(access);
   }
 
   if(regchptr->restricted && level < MEMBER_FLAG)
@@ -1860,7 +2015,7 @@ cs_on_client_join(va_list args)
     return pass_callback(cs_join_hook, source_p, name);
   }
 
-  if(IsChanop(source_p, chptr) && level < CHANOP_FLAG)
+  if(!regchptr->leaveops && (IsChanop(source_p, chptr) && level < CHANOP_FLAG))
   {
     deop_user(chanserv, chptr, source_p);
     reply_user(chanserv, chanserv, source_p, CS_DEOP_REGISTERED, 
@@ -1907,8 +2062,17 @@ cs_on_channel_create(va_list args)
 {
   struct Channel *chptr = va_arg(args, struct Channel *);
 
-  if(chptr->regchan != NULL && chptr->regchan->mlock != NULL)
-    set_mode_lock(chanserv, chptr->chname, NULL, chptr->regchan->mlock, NULL);
+  if(chptr->regchan != NULL)
+  {
+    if(chptr->regchan->mlock != NULL)
+      set_mode_lock(chanserv, chptr->chname, NULL, chptr->regchan->mlock, NULL);
+
+    if(!IsConnecting(me.uplink))
+    {
+      send_topic(chanserv, chptr, find_client(chanserv->name), 
+          chptr->regchan->topic); 
+    }
+  }
 
   return pass_callback(cs_channel_create_hook, chptr);
 }
@@ -1941,21 +2105,6 @@ cs_on_channel_destroy(va_list args)
   return pass_callback(cs_channel_destroy_hook, chan);
 }
 
-/**
- * @brief CS Callback when a nick is dropped
- * @param args 
- * @return pass_callback(self, char *)
- * When a Nick is dropped
- * - we need to make sure theres no Channel left with the nick as eounder
- */
-static void *
-cs_on_nick_drop(va_list args)
-{
-  char *nick = va_arg(args, char *);
-
-  return pass_callback(cs_on_nick_drop_hook, nick);
-}
-
 static void *
 cs_on_topic_change(va_list args)
 {
@@ -1972,13 +2121,55 @@ cs_on_topic_change(va_list args)
   {
     if(regchptr->topic != NULL)
     {
-      if(chan->topic == NULL || ircncmp(chan->topic, regchptr->topic,
-            LIBIO_MAX(strlen(chan->topic), strlen(regchptr->topic))) != 0)
+      if(chan->topic == NULL || 
+          ircncmp(chan->topic, regchptr->topic, TOPICLEN) != 0)
       {
         send_topic(chanserv, chan, find_client(chanserv->name), regchptr->topic); 
       }
     }
+    else
+      send_topic(chanserv, chan, find_client(chanserv->name), NULL);
+  }
+  else
+  {
+    /* Don't set empty topics on burst */
+    if(chan->topic != NULL || !IsConnecting(me.uplink))
+      m_set_string(chanserv, NULL, chan->chname, "TOPIC", SET_CHAN_TOPIC, 
+          chan->topic, &regchptr->topic, 2); /* 2 so we set it rather than display*/
   }
 
   return pass_callback(cs_on_topic_change_hook, chan, setter);
 }
+
+static void *
+cs_on_burst_done(va_list args)
+{
+  dlink_node *ptr;
+  struct Client *chanserv_client = find_client(chanserv->name);
+
+  DLINK_FOREACH(ptr, global_channel_list.head)
+  {
+    struct Channel *chptr = (struct Channel *)ptr->data;
+    struct RegChannel *regchptr = chptr->regchan;
+
+    if(regchptr == NULL)
+      continue;
+
+    if(regchptr->topic_lock)
+    {
+      if((regchptr->topic == NULL && !EmptyString(chptr->topic)) || 
+          (regchptr->topic != NULL && (EmptyString(chptr->topic) ||
+           ircncmp(chptr->topic, regchptr->topic, TOPICLEN) != 0)))
+      {
+        send_topic(chanserv, chptr, chanserv_client, regchptr->topic);
+      }
+    }
+    else
+    {
+      if(EmptyString(chptr->topic) && regchptr->topic != NULL)
+        send_topic(chanserv, chptr, chanserv_client, regchptr->topic);
+    }
+  }
+
+  return pass_callback(cs_on_burst_done_hook);
+}
