@@ -23,7 +23,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: channel_mode.c 874 2007-04-25 18:04:07Z tjfontaine $
+ *  $Id: channel_mode.c 1043 2007-05-18 09:24:56Z swalsh $
  */
 
 #include "stdinc.h"
@@ -795,12 +795,6 @@ chm_voice(struct Client *client_p, struct Client *source_p,
   struct Client *targ_p;
   struct Membership *member;
 
-  if (alev < CHACCESS_HALFOP)
-  {
-    *errors |= SM_ERR_NOOPS;
-    return;
-  }
-
   if ((dir == MODE_QUERY) || parc <= *parn)
     return;
 
@@ -1073,6 +1067,12 @@ init_channel_modes(void)
                                         get_channel_access);
 }
 
+void
+cleanup_channel_modes()
+{
+  unregister_callback(channel_access_cb);
+}
+
 /* void set_channel_mode(struct Client *client_p, struct Client *source_p,
  *               struct Channel *chptr, int parc, char **parv,
  *               char *chname)
