@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: dbm.c 915 2007-05-03 16:28:18Z stu $
+ *  $Id: dbm.c 1043 2007-05-18 09:24:56Z swalsh $
  */
 
 #include "stdinc.h"
@@ -35,7 +35,7 @@ static void expire_sentmail(void *);
 static void expire_akills(void *);
 
 query_t queries[QUERY_COUNT] = { 
-  { GET_FULL_NICK, "SELECT account.id, primary_nick, nickname.id, nickname.nick, "
+  { GET_FULL_NICK, "SELECT account.id, primary_nick, nickname.id, "
     "(SELECT nick FROM nickname WHERE nickname.id=account.primary_nick), "
     "password, salt, url, email, cloak, flag_enforce, flag_secure, "
     "flag_verified, flag_cloak_enabled, flag_admin, flag_email_verified, "
@@ -45,13 +45,14 @@ query_t queries[QUERY_COUNT] = {
     "lower(nick) = lower(?v)", NULL, QUERY },
   { GET_NICK_FROM_ACCID, "SELECT nick from account, nickname WHERE account.id=?d AND "
     "account.primary_nick=nickname.id", NULL, QUERY },
+  { GET_NICK_FROM_NICKID, "SELECT nick from nickname WHERE id=?d", NULL, QUERY },
   { GET_ACCID_FROM_NICK, "SELECT account_id from nickname WHERE lower(nick)=lower(?v)", NULL, QUERY },
   { GET_NICKID_FROM_NICK, "SELECT id from nickname WHERE lower(nick)=lower(?v)", NULL, QUERY },
   { INSERT_ACCOUNT, "INSERT INTO account (primary_nick, password, salt, email, reg_time) VALUES "
     "(?d, ?v, ?v, ?v, ?d)", NULL, EXECUTE },
   { INSERT_NICK, "INSERT INTO nickname (id, nick, account_id, reg_time, last_seen) VALUES "
     "(?d, ?v, ?d, ?d, ?d)", NULL, EXECUTE },
-  { DELETE_NICK, "DELETE FROM nickname WHERE lower(nick)=lower(?v)", NULL, EXECUTE },
+  { DELETE_NICK, "DELETE FROM nickname WHERE id=?d", NULL, EXECUTE },
   { DELETE_ACCOUNT, "DELETE FROM account WHERE id=?d", NULL, EXECUTE },
   { INSERT_NICKACCESS, "INSERT INTO account_access (account_id, entry) VALUES(?d, ?v)", 
     NULL, EXECUTE },
@@ -60,14 +61,19 @@ query_t queries[QUERY_COUNT] = {
     "account.primary_nick = nickname.id ORDER BY nick", NULL, QUERY },
   { GET_AKILLS, "SELECT akill.id, setter, mask, reason, time, duration FROM akill", 
     NULL, QUERY },
-  { GET_CHAN_ACCESSES, "SELECT id, channel_id, account_id, level FROM channel_access WHERE "
-    "channel_id=?d", NULL, QUERY },
-  { GET_CHANID_FROM_CHAN, "SELECT id from channel WHERE lower(channel)=lower(?v)", NULL, QUERY },
+  { GET_CHAN_ACCESSES, "SELECT channel_access.id, channel_access.channel_id, "
+      "channel_access.account_id, channel_access.level FROM "
+      "channel_access JOIN account ON "
+      "channel_access.account_id=account.id JOIN nickname ON "
+      "account.primary_nick=nickname.id WHERE channel_id=?d "
+      "ORDER BY lower(nickname.nick)", NULL, QUERY },
+  { GET_CHANID_FROM_CHAN, "SELECT id from channel WHERE "
+      "lower(channel)=lower(?v)", NULL, QUERY },
   { GET_FULL_CHAN, "SELECT id, channel, description, entrymsg, reg_time, "
       "flag_private, flag_restricted, flag_topic_lock, flag_verbose, "
       "flag_autolimit, flag_expirebans, flag_floodserv, flag_autoop, "
-      "flag_autovoice, url, email, topic, mlock FROM channel WHERE "
-      "lower(channel)=lower(?v)", NULL, QUERY },
+      "flag_autovoice, flag_leaveops, url, email, topic, mlock FROM "
+      "channel WHERE lower(channel)=lower(?v)", NULL, QUERY },
   { INSERT_CHAN, "INSERT INTO channel (channel, description, reg_time, last_used) "
     "VALUES(?v, ?v, ?d, ?d)", NULL, EXECUTE },
   { INSERT_CHANACCESS, "INSERT INTO channel_access (account_id, channel_id, level) VALUES "
@@ -133,6 +139,7 @@ query_t queries[QUERY_COUNT] = {
   { SET_CHAN_FLOODSERV, "UPDATE channel SET flag_floodserv=?B WHERE id=?d", NULL, EXECUTE },
   { SET_CHAN_AUTOOP, "UPDATE channel SET flag_autoop=?B WHERE id=?d", NULL, EXECUTE },
   { SET_CHAN_AUTOVOICE, "UPDATE channel SET flag_autovoice=?B WHERE id=?d", NULL, EXECUTE },
+  { SET_CHAN_LEAVEOPS, "UPDATE channel SET flag_leaveops=?B WHERE id=?d", NULL, EXECUTE },
   { INSERT_FORBID, "INSERT INTO forbidden_nickname (nick) VALUES (?v)", NULL, EXECUTE },
   { GET_FORBID, "SELECT nick FROM forbidden_nickname WHERE lower(nick)=lower(?v)",
     NULL, QUERY },
@@ -163,8 +170,10 @@ query_t queries[QUERY_COUNT] = {
   { GET_CHAN_MASTER_COUNT, "SELECT COUNT(id) FROM channel_access WHERE channel_id=?d AND level=?d",
     NULL, QUERY },
   { GET_NICK_LINKS, "SELECT nick FROM nickname WHERE account_id=?d", NULL, QUERY },
-  { GET_NICK_CHAN_INFO, "SELECT channel, level FROM channel, channel_access WHERE channel.id="
-    "channel_access.channel_id AND channel_access.account_id=?d", NULL, QUERY },
+  { GET_NICK_CHAN_INFO, "SELECT channel, level FROM "
+    "channel, channel_access WHERE "
+      "channel.id=channel_access.channel_id AND channel_access.account_id=?d "
+      "ORDER BY lower(channel.channel)", NULL, QUERY },
   { GET_CHAN_MASTERS, "SELECT nick FROM account, nickname, channel_access WHERE channel_id=?d "
     "AND level=4 AND channel_access.account_id=account.id AND "
       "account.primary_nick=nickname.id", NULL, QUERY },
@@ -226,6 +235,8 @@ init_db()
   Database.yada = yada_init(dbstr, 0);
   MyFree(dbstr);
 
+  MyFree(dbstr);
+
   snprintf(logpath, LOG_BUFSIZE, "%s/%s", LOGDIR, Logging.sqllog);
   if(db_log_fb == NULL)
   {
@@ -240,8 +251,19 @@ init_db()
 void
 cleanup_db()
 {
-  Database.yada->disconnect(Database.yada);
-//  Database.yada->destroy(Database.yada);
+  int i;
+
+  if(Database.yada != NULL)
+  {
+    Database.yada->disconnect(Database.yada);
+    for(i = 0; i < QUERY_COUNT; i++)
+    {
+      query_t *query = &queries[i];
+      Free(query->rc);
+    }
+    Database.yada->destroy(Database.yada);
+  }
+  fbclose(db_log_fb);
 }
 
 void
@@ -284,7 +306,7 @@ db_load_driver()
       continue;
     query->rc = Prepare((char*)query->name, 0);
     if(query->rc == NULL)
-      ilog(L_CRIT, "Prepare: %d Failed: %s\n", i, Database.yada->errmsg);
+      ilog(L_CRIT, "Prepare: %d Failed: %s", i, Database.yada->errmsg);
   }
 
   eventAdd("Expire sent mail", expire_sentmail, NULL, 60); 
@@ -316,7 +338,7 @@ db_try_reconnect()
           continue;
         query->rc = Prepare((char*)query->name, 0);
         if(query->rc == NULL)
-          ilog(L_CRIT, "Prepare: %d Failed: %s\n", i, Database.yada->errmsg);
+          ilog(L_CRIT, "Prepare: %d Failed: %s", i, Database.yada->errmsg);
       }
       return;
     }
@@ -349,7 +371,7 @@ db_try_reconnect()
       db_try_reconnect();                                             \
       db_log("Query failed because server went away, reconnected.");  \
     }                                                                 \
-    ilog(L_CRIT, "db_query: %d Failed: %s\n", __id, Database.yada->errmsg); \
+    db_log(L_CRIT, "db_query: %d Failed: %s", __id, Database.yada->errmsg); \
   }                                                                   \
                                                                       \
   ret = __result;                                                     \
@@ -385,7 +407,7 @@ db_find_nick(const char *nick)
 {
   yada_rc_t *rc, *brc;
   struct Nick *nick_p;
-  char *retnick, *retpass, *retcloak, *retsalt, *retrealnick;
+  char *retnick, *retpass, *retcloak, *retsalt;
 
   assert(nick != NULL);
 
@@ -396,8 +418,8 @@ db_find_nick(const char *nick)
  
   nick_p = MyMalloc(sizeof(struct Nick));
  
-  brc = Bind("?d?d?d?ps?ps?ps?ps?ps?ps?ps?B?B?B?B?B?B?B?d?ps?ps?ps?d?d?d?d",
-    &nick_p->id, &nick_p->pri_nickid, &nick_p->nickid, &retrealnick, &retnick, 
+  brc = Bind("?d?d?d?ps?ps?ps?ps?ps?ps?B?B?B?B?B?B?B?d?ps?ps?ps?d?d?d?d",
+    &nick_p->id, &nick_p->pri_nickid, &nick_p->nickid, &retnick, 
     &retpass, &retsalt, &nick_p->url, &nick_p->email, &retcloak, 
     &nick_p->enforce, &nick_p->secure, &nick_p->verified, &nick_p->cloak_on, 
     &nick_p->admin, &nick_p->email_verified, &nick_p->priv, &nick_p->language, 
@@ -416,7 +438,6 @@ db_find_nick(const char *nick)
 
   assert(retnick != NULL);
   strlcpy(nick_p->nick, retnick, sizeof(nick_p->nick));
-  strlcpy(nick_p->real_nick, retrealnick, sizeof(nick_p->nick));
   strlcpy(nick_p->pass, retpass, sizeof(nick_p->pass));
   strlcpy(nick_p->salt, retsalt, sizeof(nick_p->salt));
   if(retcloak)
@@ -464,6 +485,34 @@ db_get_nickname_from_id(unsigned int id)
   return retnick;
 }
 
+char *
+db_get_nickname_from_nickid(unsigned int id)
+{
+  yada_rc_t *rc, *brc; 
+  char *retnick; 
+
+  db_query(rc, GET_NICK_FROM_NICKID, id);
+
+  if(rc == NULL)
+    return NULL;
+
+  brc = Bind("?ps", &retnick);
+  if(Fetch(rc, brc) == 0)
+  {
+    db_log("db_get_nickname_from_nickid: %d not found.\n", id);
+    Free(brc);
+    Free(rc);
+    return NULL;
+  }
+
+  DupString(retnick, retnick);
+
+  Free(brc);
+  Free(rc);
+
+  return retnick;
+}
+
 unsigned int
 db_get_id_from_name(const char *name, unsigned int type)
 {
@@ -604,7 +653,7 @@ db_forbid_nick(const char *n)
 
   if((nick = db_find_nick(n)) != NULL)
   {
-    db_delete_nick(nick->id, nick->nickid, nick->nick);
+    db_delete_nick(nick->id, nick->nickid, nick->pri_nickid);
     free_nick(nick);
   }
 
@@ -712,30 +761,37 @@ db_set_nick_master(unsigned int accid, const char *newnick)
 }
 
 int
-db_delete_nick(unsigned int accid, unsigned int nickid, const char *nick)
+db_delete_nick(unsigned int accid, unsigned int nickid, unsigned int priid)
 {
   int ret;
   unsigned int newid;
 
   TransBegin();
 
-  newid = db_fix_link(accid, nickid);
-
-  if(newid == -1)
+  if(priid == nickid)
   {
-    db_exec(ret, DELETE_NICK, nick);
-    if(ret != -1)
+    newid = db_fix_link(accid, nickid);
+
+    if(newid == -1)
     {
-      db_exec(ret, DELETE_ACCOUNT_CHACCESS, accid);
+      db_exec(ret, DELETE_NICK, nickid);
       if(ret != -1)
-        db_exec(ret, DELETE_ACCOUNT, accid);
+      {
+        db_exec(ret, DELETE_ACCOUNT_CHACCESS, accid);
+        if(ret != -1)
+          db_exec(ret, DELETE_ACCOUNT, accid);
+      }
+    }
+    else
+    {
+      db_exec(ret, SET_NICK_MASTER, newid, accid);
+      if(ret != -1)
+        db_exec(ret, DELETE_NICK, nickid);
     }
   }
   else
   {
-    db_exec(ret, SET_NICK_MASTER, newid, accid);
-    if(ret != -1)
-      db_exec(ret, DELETE_NICK, nick);
+    db_exec(ret, DELETE_NICK, nickid);
   }
 
   if(ret == -1)
@@ -746,8 +802,6 @@ db_delete_nick(unsigned int accid, unsigned int nickid, const char *nick)
 
   if(TransCommit() != 0)
     return FALSE;
-
-  execute_callback(on_nick_drop_cb, nick);
  
   return TRUE;
 }
@@ -955,10 +1009,18 @@ db_list_first(unsigned int type, unsigned int param, void **entry)
   db_query(rc, query, param);
 
   if(rc == NULL || brc == NULL)
+  {
+    if(brc != NULL)
+      Free(brc);
     return NULL;
+  }
 
   if(Fetch(rc, brc) == 0)
+  {
+    Free(brc);
+    Free(rc);
     return NULL;
+  }
 
   result = MyMalloc(sizeof(struct DBResult));
 
@@ -1224,13 +1286,13 @@ db_find_chan(const char *channel)
  
   channel_p = MyMalloc(sizeof(struct RegChannel));
  
-  brc = Bind("?d?ps?ps?ps?d?B?B?B?B?B?B?B?B?B?ps?ps?ps?ps",
+  brc = Bind("?d?ps?ps?ps?d?B?B?B?B?B?B?B?B?B?B?ps?ps?ps?ps",
       &channel_p->id, &retchan, &channel_p->description, &channel_p->entrymsg, 
       &channel_p->regtime, &channel_p->priv, &channel_p->restricted,
       &channel_p->topic_lock, &channel_p->verbose, &channel_p->autolimit,
       &channel_p->expirebans, &channel_p->floodserv, &channel_p->autoop,
-      &channel_p->autovoice, &channel_p->url, &channel_p->email, 
-      &channel_p->topic, &channel_p->mlock);
+      &channel_p->autovoice, &channel_p->leaveops, &channel_p->url, 
+      &channel_p->email, &channel_p->topic, &channel_p->mlock);
 
   if(Fetch(rc, brc) == 0)
   {
