@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: interface.c 919 2007-05-03 20:30:53Z stu $
+ *  $Id: interface.c 1043 2007-05-18 09:24:56Z swalsh $
  */
 
 #include "stdinc.h"
@@ -62,6 +62,7 @@ struct Callback *on_channel_destroy_cb;
 struct Callback *on_topic_change_cb;
 struct Callback *on_privmsg_cb;
 struct Callback *on_notice_cb;
+struct Callback *on_burst_done_cb;
 
 struct LanguageFile ServicesLanguages[LANG_LAST];
 struct ModeList *ServerModeList;
@@ -97,7 +98,6 @@ init_interface()
   on_quit_cb          = register_callback("Propagate QUIT", NULL);
   on_umode_change_cb  = register_callback("Propagate UMODE", NULL);
   on_cmode_change_cb  = register_callback("Propagate CMODE", NULL);
-  on_quit_cb          = register_callback("Propagate SQUIT", NULL);
   on_identify_cb      = register_callback("Identify Callback", NULL);
   on_newuser_cb       = register_callback("New user coming to us", NULL);
   on_channel_created_cb = register_callback("Channel is being created", NULL);
@@ -105,6 +105,8 @@ init_interface()
   on_topic_change_cb  = register_callback("Topic changed", NULL);
   on_privmsg_cb       = register_callback("Privmsg for channel received", NULL);
   on_notice_cb        = register_callback("Notice for channel received", NULL);
+  on_burst_done_cb    = register_callback("Notification that burst is complete", 
+      NULL);
 
   load_language(ServicesLanguages, "services.en");
 }
@@ -113,6 +115,42 @@ void
 cleanup_interface()
 {
   BlockHeapDestroy(services_heap);
+  unregister_callback(send_newuser_cb);
+  unregister_callback(send_privmsg_cb);
+  unregister_callback(send_notice_cb);
+  unregister_callback(send_gnotice_cb);
+  unregister_callback(send_umode_cb);
+  unregister_callback(send_cloak_cb);
+  unregister_callback(send_nick_cb);
+  unregister_callback(send_akill_cb);
+  unregister_callback(send_unakill_cb);
+  unregister_callback(send_kick_cb);
+  unregister_callback(send_cmode_cb);
+  unregister_callback(send_invite_cb);
+  unregister_callback(send_topic_cb);
+  unregister_callback(send_kill_cb);
+  unregister_callback(send_resv_cb);
+  unregister_callback(send_unresv_cb);
+  unregister_callback(send_newserver_cb);
+  unregister_callback(send_join_cb);
+  unregister_callback(send_part_cb);
+  unregister_callback(send_nosuchsrv_cb);
+  unregister_callback(on_nick_change_cb);
+  unregister_callback(on_join_cb);
+  unregister_callback(on_part_cb);
+  unregister_callback(on_quit_cb);
+  unregister_callback(on_umode_change_cb);
+  unregister_callback(on_cmode_change_cb);
+  unregister_callback(on_identify_cb);
+  unregister_callback(on_newuser_cb);
+  unregister_callback(on_channel_created_cb);
+  unregister_callback(on_channel_destroy_cb);
+  unregister_callback(on_topic_change_cb);
+  unregister_callback(on_privmsg_cb);
+  unregister_callback(on_notice_cb);
+  unregister_callback(on_burst_done_cb);
+
+  unload_languages(ServicesLanguages);
 }
 
 struct Service *
@@ -387,8 +425,12 @@ kill_user(struct Service *service, struct Client *client, const char *reason)
     ilog(L_DEBUG, "Was going to kill: %s (%s)", client->name, reason);
   else
   {
-    execute_callback(send_kill_cb, me.uplink, service, client, reason);
-    exit_client(client, &me, reason);
+    if(!IsKilled(client))
+    {
+      execute_callback(send_kill_cb, me.uplink, service, client, reason);
+      exit_client(client, &me, reason);
+      SetKilled(client);
+    }
   }
 }
 
@@ -399,6 +441,13 @@ akill_add(struct Service *service, struct Client *client, const char* mask,
   struct ServiceBan *akill;
   int ret = 0;
 
+  if(!valid_wild_card(mask))
+  {
+    ilog(L_NOTICE, "%s tried to add an akill(%s) that was too wild!", 
+        client->nickname != NULL ? client->nickname->nick : "Services", mask);
+    return NULL;
+  }
+
   akill = MyMalloc(sizeof(struct ServiceBan));
 
   akill->type = AKILL_BAN;
@@ -774,13 +823,16 @@ replace_string(char *str, const char *value)
 int
 check_list_entry(unsigned int type, unsigned int id, const char *value)
 {
-  struct AccessEntry *entry;
+  struct AccessEntry *entry = NULL;
   void *ptr, *first;
 
   first = ptr = db_list_first(type, id, (void**)&entry);
 
   if(ptr == NULL)
+  {
+    MyFree(entry);
     return FALSE;
+  }
 
   while(ptr != NULL)
   {
@@ -796,8 +848,10 @@ check_list_entry(unsigned int type, unsigned int id, const char *value)
     ilog(L_DEBUG, "check_list_entry: Not Found match: %s %s", entry->value, 
         value);
     MyFree(entry);
+    entry = NULL;
     ptr = db_list_next(ptr, type, (void**)&entry);
   }
+  MyFree(entry);
   db_list_done(first);
   return FALSE;
 }
@@ -1027,7 +1081,7 @@ set_mode_lock(struct Service *service, const char *channel,
   {
     /*
      * l and k are both special cases because they arent listed in the
-     * protocol's mode list because they take paramters, but we can MLOCK
+     * protocol's mode list because they take parameters, but we can MLOCK
      * them.
      */
     if(mode != '+' && mode != '-' && mode != 'l' && mode != 'k')
@@ -1202,7 +1256,7 @@ set_mode_lock(struct Service *service, const char *channel,
     strcpy(chptr->mode.key, key);
  
   /* 
-   * Set up the modestring and paramter(s) and set them. This could probably
+   * Set up the modestring and parameter(s) and set them. This could probably
    * be written a little better.
    */
   if(limit > 0 && key[0] != '\0')
@@ -1247,6 +1301,7 @@ free_regchan(struct RegChannel *regchptr)
   MyFree(regchptr->url);
   MyFree(regchptr->email);
   MyFree(regchptr->topic);
+  MyFree(regchptr->mlock);
   mqueue_hash_free(regchptr->flood_hash, &regchptr->flood_list);
   regchptr->flood_hash = NULL;
   mqueue_free(regchptr->gqueue);
@@ -1279,6 +1334,8 @@ free_serviceban(struct ServiceBan *ban)
 void
 free_chanaccess(struct ChanAccess *access)
 {
+  if(access == NULL)
+    return;
 	ilog(L_DEBUG, "Freeing chanaccess %p for channel %d", access, access->channel);
 	MyFree(access);
 }
