@@ -23,7 +23,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: parse.c 793 2007-04-07 13:53:02Z stu $
+ *  $Id: parse.c 1043 2007-05-18 09:24:56Z swalsh $
  */
 
 #include "stdinc.h"
@@ -316,7 +316,7 @@ handle_services_command(struct ServiceMessage *pmptr,
   struct Channel *chptr;
   struct RegChannel *regchptr = NULL;
   struct ChanAccess *access;
-  unsigned int level;
+  unsigned int level = 0;
 
   if(i < mptr->parameters)
   {
@@ -382,6 +382,8 @@ handle_services_command(struct ServiceMessage *pmptr,
     if(from->access < IDENTIFIED_FLAG && mptr->access == CHIDENTIFIED_FLAG)
     {
       reply_user(service, NULL, from, SERV_NOT_IDENTIFIED, from->name);
+      if(regchptr != chptr->regchan)
+        free_regchan(regchptr);
       return;
     }
 
@@ -404,13 +406,25 @@ handle_services_command(struct ServiceMessage *pmptr,
 
     if(level < mptr->access)
     {
-      reply_user(service, NULL, from, SERV_NO_ACCESS_CHAN, mptr->cmd,
-          regchptr->channel);
+      if(level > CHUSER_FLAG)
+        reply_user(service, NULL, from, SERV_NO_ACCESS_CHAN, mptr->cmd,
+            regchptr->channel);
+      else
+        reply_user(service, NULL, from, SERV_NO_ACCESS_CHAN_ID, mptr->cmd,
+            regchptr->channel);
+      if(regchptr != chptr->regchan)
+        free_regchan(regchptr);
       return;
     }
 
     if(chptr != NULL)
+    {
+      if(chptr->regchan != regchptr)
+        free_regchan(chptr->regchan);
       chptr->regchan = regchptr;
+    }
+    else
+      free_regchan(regchptr);
   }
   else
   {
@@ -773,7 +787,7 @@ recurse_help_messages(struct Service *service, struct Client *client,
 
   if (mtree->msg != NULL && !(mtree->msg->flags & SFLG_ALIAS))
   {
-    if(mtree->msg->flags & SFLG_CHANARG || 
+    if(mtree->msg->flags & SFLG_CHANARG || mtree->msg->access < OPER_FLAG ||
         client->access >= mtree->msg->access)
       reply_user(service, service, client, mtree->msg->help_short, 
           mtree->msg->cmd);
@@ -803,18 +817,18 @@ do_serv_help_messages(struct Service *service, struct Client *client)
 }
 
 static void
-recurse_clear_messages(struct ServiceMessageTree *mtree)
+recurse_clear_messages(struct ServiceMessageTree *mtree) 
 {
   int i;
 
-  if (mtree->msg != NULL && !(mtree->msg->flags & SFLG_ALIAS))
-  {
-    serv_del_msg_element(mtree, mtree->msg->cmd);
-  }
   for (i = 0; i < MAXPTRLEN; i++)
   {
     if (mtree->pointers[i] != NULL)
+    {
       recurse_clear_messages(mtree->pointers[i]);
+      MyFree(mtree->pointers[i]);
+      mtree->pointers[i] = NULL;
+    }
   }
 }
 
@@ -827,7 +841,11 @@ serv_clear_messages(struct Service *service)
   for (i = 0; i < MAXPTRLEN; i++)
   {
     if (mtree->pointers[i] != NULL)
+    {
       recurse_clear_messages(mtree->pointers[i]);
+      MyFree(mtree->pointers[i]);
+      mtree->pointers[i] = NULL;
+    }
   }
 }
 
